#include "stdafx.h"
#include "TaskSimulate.h"

/*
Simulation dialogue handler 

Contents : 
	CVwSimulate - Options dialog 	
	VwSimulateDlgProc - message pump for options dialog 
	OpenVwSimulate( HWND hDlg, CData * ) - external entry point to open task options dialog 
*/

BOOL CALLBACK VwSimulateDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) ;

//-- Global data 
char * g_szTitleSimulate = "Data Simulation";

//-----------------------------------------------------------------------------
// dialog for options edit 
class CVwSimulate : public CDialog { 
	private : 
		//-- Controls 
		CDlgEdit m_edtSegments;
		CDlgEdit m_edtPointCount;
		CDlgCheck m_chkRandomSeed;
		CDlgEdit m_edtRandomSeed;
		CDlgEdit m_edtChannelCount;
		CDlgEdit m_edtSampleRate;
		CDlgControl m_lstStimulation;
		CDlgEdit m_edtFileName;
		CDlgCheck m_chkRemoveDwells;
		
	public : 		
		COptionsSimulate * m_pConfig;		// ptr to structure to store control values when done

		CVwSimulate() { 
			m_iID = IDD_TASK_SIMULATE;
			m_pConfig = NULL;
			}
	
		~CVwSimulate() { 
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			m_edtSegments.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_SEGMENTS ), "Segments (traces) of data to generate" );
			m_edtPointCount.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_POINTCOUNT ), "Points per segment" );
			m_chkRandomSeed.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_RANDOMSEED ), "Initialize the random number generator with th supplied specific value" );
			m_edtRandomSeed.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_RANDOMSEED ), "Specific value to use to initialize the random number generator" );
			m_edtSampleRate.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_SAMPLERATE ), "Sample Rate - Samples per second for simulated data." );
			m_edtChannelCount.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_CHANNELCOUNT ), "Number of Ion channels to simulate" );
			m_lstStimulation.SetHandle( GetDlgItem( m_hDlg, IDC_LST_STIMULATION ), "List of stimulation protocols or data corresponding to named parameters in the model" );
			m_edtFileName.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_FILENAME ), "Output file to store simulated data (file will be overwritten)." );
			m_chkRemoveDwells.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_REMOVEDWELLS ), "Remove dwells that do not cross a sample point." );
			}

		//-- Set all control contents and disable unused rows
		void SetControlValues( ) { 
			m_edtSegments.SetTextInt( m_pConfig->m_iSegments );
			m_edtPointCount.SetTextInt( m_pConfig->m_iPointCount );
			m_chkRandomSeed.SetCheck( m_pConfig->m_bRandomSeed );
			m_edtRandomSeed.SetTextInt( m_pConfig->m_iRandomSeed );
			m_edtSampleRate.SetTextInt( m_pConfig->m_iSampleRate );
			m_edtChannelCount.SetTextInt( m_pConfig->m_iChannelCount );
			m_chkRemoveDwells.SetCheck( m_pConfig->m_bRemoveDwells );
			//m_lstStimulation
			//TODO - populate stim list with names from the provided model
			m_edtFileName.SetText( m_pConfig->m_cFileName.Buffer() );
			}

		//-----------------------------------------------------------------------------
		bool GetControlValues(  ) { 
			m_pConfig->m_iSegments = m_edtSegments.GetTextInt();
			m_pConfig->m_iPointCount = m_edtPointCount.GetTextInt(  );
			m_pConfig->m_bRandomSeed = m_chkRandomSeed.GetCheck(  );
			m_pConfig->m_iRandomSeed = m_edtRandomSeed.GetTextInt(  );
			m_pConfig->m_iChannelCount = m_edtChannelCount.GetTextInt(  );
			m_edtFileName.GetText( &m_pConfig->m_cFileName );
			m_pConfig->m_iSampleRate = m_edtSampleRate.GetTextInt();
			m_pConfig->m_bRemoveDwells = m_chkRemoveDwells.GetCheck();
			m_pConfig->m_bRemoveDwells = m_chkRemoveDwells.GetCheck();
			//m_lstStimulation
			//TODO - get stimulation entries

			//-- TODO - data validations
			if( false ) { 
				MessageBox( m_hDlg, "...", g_szTitleSimulate, MB_OK );
				return false;
				}
			
			return true;
			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [CANCEL]  
		void OnBtnCancel( ) { 
			EndDialog( m_hDlg, IDCANCEL );
			}
			
		//-- [SAVE] 
		void OnSave( ) { 
			if( GetControlValues() ) 
				EndDialog( m_hDlg, IDOK );
			}
			
	};

//-----------------------------------------------------------------------------
CVwSimulate * g_pVwSimulate;		// Modal - one and only instance

//-----------------------------------------------------------------------------
void OpenVwSimulate( HWND hDlg, CModel * pModel ) { 
	//-- create structure to pass option to / from dialogue.
	COptionsSimulate * pConfig = new COptionsSimulate;
	pConfig->m_iSegments=10;
	pConfig->m_iPointCount=1000;
	pConfig->m_bRandomSeed=false;
	pConfig->m_iRandomSeed=0;
	pConfig->m_iChannelCount=1;
	pConfig->m_iSampleRate=10000;
	pConfig->m_cFileName="C:\\Temp\\Simdata.qdf";
	//TODO pConfig->m_aStimulation = ... 
	// Note : pConfig->m_oModel is not set, relevant options are in Stimulation list.
	//		  It will be set later when it is needed for simulation.

	pConfig->RegistryLoad();

	//-- create dialogue to get task options 	
	g_pVwSimulate = new CVwSimulate;
	g_pVwSimulate->m_pConfig=pConfig;
	
	//-- present dialogue, execute if [OK]
	if( DialogBox( GetInstance(), (LPCTSTR)IDD_TASK_SIMULATE, hDlg, (DLGPROC)VwSimulateDlgProc ) == IDOK ) { 
		pConfig->RegistrySave();
		
		//-- Open the task & progress view 	
		CTaskSimulate * pTaskSimulate = new CTaskSimulate( );
		pTaskSimulate->m_pOptionsSimulate = pConfig;		// transfer ownership to task (for later delete)
		pTaskSimulate->m_pOptionsSimulate->m_oModel = *pModel;
		pTaskSimulate->m_hwndDataView = OpenDataView( pConfig->m_cFileName.ConstBuffer() );
		PostMessage( pTaskSimulate->m_hwndDataView, WM_DATA_CREATESIM, (WPARAM)pConfig, 0 );	

		//-- Open Task thread 		
		HWND hVwTask = pTaskSimulate->CreateTaskWindow( GetClientWin(), GetChildClassName() );
		pTaskSimulate->m_hThread = CreateThread( NULL, 0, TaskSimulate, pTaskSimulate, 0, NULL );
		} 
	else 
		delete pConfig;
		
	}

//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK VwSimulateDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwSimulate->m_hDlg=hDlg;			

			//-- Clear Existing  Values
			g_pVwSimulate->InitializeControls();
			g_pVwSimulate->SetControlValues();
			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 

			switch( wmId ) { 
				case IDOK :					//-- [OK]
					g_pVwSimulate->OnSave( );
					break;

				case IDCANCEL :				//-- [CANCEL]
					g_pVwSimulate->OnBtnCancel( );
					break;
				}
				
			return TRUE;

		case WM_PAINT : 
			/*
			{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hDlg, &ps);
							
			//----- Manual Drawing 
				
			//----- Done
			EndPaint(hDlg, &ps);
			}
			*/
			return false;

		case WM_DESTROY : 
			delete g_pVwSimulate;
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 

